# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import JistBrainMgdmSegmentation


def test_JistBrainMgdmSegmentation_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inAdjust=dict(argstr='--inAdjust %s', ),
        inAtlas=dict(
            argstr='--inAtlas %s',
            extensions=None,
        ),
        inCompute=dict(argstr='--inCompute %s', ),
        inCurvature=dict(argstr='--inCurvature %f', ),
        inData=dict(argstr='--inData %f', ),
        inFLAIR=dict(
            argstr='--inFLAIR %s',
            extensions=None,
        ),
        inMP2RAGE=dict(
            argstr='--inMP2RAGE %s',
            extensions=None,
        ),
        inMP2RAGE2=dict(
            argstr='--inMP2RAGE2 %s',
            extensions=None,
        ),
        inMPRAGE=dict(
            argstr='--inMPRAGE %s',
            extensions=None,
        ),
        inMax=dict(argstr='--inMax %d', ),
        inMin=dict(argstr='--inMin %f', ),
        inOutput=dict(argstr='--inOutput %s', ),
        inPV=dict(
            argstr='--inPV %s',
            extensions=None,
        ),
        inPosterior=dict(argstr='--inPosterior %f', ),
        inSteps=dict(argstr='--inSteps %d', ),
        inTopology=dict(argstr='--inTopology %s', ),
        null=dict(argstr='--null %s', ),
        outLevelset=dict(
            argstr='--outLevelset %s',
            hash_files=False,
        ),
        outPosterior2=dict(
            argstr='--outPosterior2 %s',
            hash_files=False,
        ),
        outPosterior3=dict(
            argstr='--outPosterior3 %s',
            hash_files=False,
        ),
        outSegmented=dict(
            argstr='--outSegmented %s',
            hash_files=False,
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = JistBrainMgdmSegmentation.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_JistBrainMgdmSegmentation_outputs():
    output_map = dict(
        outLevelset=dict(extensions=None, ),
        outPosterior2=dict(extensions=None, ),
        outPosterior3=dict(extensions=None, ),
        outSegmented=dict(extensions=None, ),
    )
    outputs = JistBrainMgdmSegmentation.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# NOTE:  This implementation has been superceeded buy the antsApplyTransform
#        implmeentation that more closely follows the strucutre and capabilities
#        of the antsApplyTransform program.  This implementation is here
#        for backwards compatibility.
"""ANTS Apply Transforms interface
"""

from builtins import range

import os
from glob import glob

from .base import ANTSCommand, ANTSCommandInputSpec
from ..base import TraitedSpec, File, traits, isdefined, OutputMultiPath
from ...utils.filemanip import split_filename


class antsIntroductionInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='-d %d',
        usedefault=True,
        desc='image dimension (2 or 3)',
        position=1)
    reference_image = File(
        exists=True,
        argstr='-r %s',
        desc='template file to warp to',
        mandatory=True,
        copyfile=True)
    input_image = File(
        exists=True,
        argstr='-i %s',
        desc='input image to warp to template',
        mandatory=True,
        copyfile=False)
    force_proceed = traits.Bool(
        argstr='-f 1',
        desc=('force script to proceed even if headers '
              'may be incompatible'))
    inverse_warp_template_labels = traits.Bool(
        argstr='-l',
        desc=('Applies inverse warp to the template labels '
              'to estimate label positions in target space (use '
              'for template-based segmentation)'))
    max_iterations = traits.List(
        traits.Int,
        argstr='-m %s',
        sep='x',
        desc=('maximum number of iterations (must be '
              'list of integers in the form [J,K,L...]: '
              'J = coarsest resolution iterations, K = '
              'middle resolution interations, L = fine '
              'resolution iterations'))
    bias_field_correction = traits.Bool(
        argstr='-n 1',
        desc=('Applies bias field correction to moving '
              'image'))
    similarity_metric = traits.Enum(
        'PR',
        'CC',
        'MI',
        'MSQ',
        argstr='-s %s',
        desc=('Type of similartiy metric used for registration '
              '(CC = cross correlation, MI = mutual information, '
              'PR = probability mapping, MSQ = mean square difference)'))
    transformation_model = traits.Enum(
        'GR',
        'EL',
        'SY',
        'S2',
        'EX',
        'DD',
        'RI',
        'RA',
        argstr='-t %s',
        usedefault=True,
        desc=('Type of transofmration model used for registration '
              '(EL = elastic transformation model, SY = SyN with time, '
              'arbitrary number of time points, S2 =  SyN with time '
              'optimized for 2 time points, GR = greedy SyN, EX = '
              'exponential, DD = diffeomorphic demons style exponential '
              'mapping, RI = purely rigid, RA = affine rigid'))
    out_prefix = traits.Str(
        'ants_',
        argstr='-o %s',
        usedefault=True,
        desc=('Prefix that is prepended to all output '
              'files (default = ants_)'))
    quality_check = traits.Bool(
        argstr='-q 1', desc='Perform a quality check of the result')


class antsIntroductionOutputSpec(TraitedSpec):
    affine_transformation = File(
        exists=True, desc='affine (prefix_Affine.txt)')
    warp_field = File(exists=True, desc='warp field (prefix_Warp.nii)')
    inverse_warp_field = File(
        exists=True, desc='inverse warp field (prefix_InverseWarp.nii)')
    input_file = File(exists=True, desc='input image (prefix_repaired.nii)')
    output_file = File(exists=True, desc='output image (prefix_deformed.nii)')


class antsIntroduction(ANTSCommand):
    """Uses ANTS to generate matrices to warp data from one space to another.

    Examples
    --------

    >>> from nipype.interfaces.ants.legacy import antsIntroduction
    >>> warp = antsIntroduction()
    >>> warp.inputs.reference_image = 'Template_6.nii'
    >>> warp.inputs.input_image = 'structural.nii'
    >>> warp.inputs.max_iterations = [30,90,20]
    >>> warp.cmdline
    'antsIntroduction.sh -d 3 -i structural.nii -m 30x90x20 -o ants_ -r Template_6.nii -t GR'

    """

    _cmd = 'antsIntroduction.sh'
    input_spec = antsIntroductionInputSpec
    output_spec = antsIntroductionOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        transmodel = self.inputs.transformation_model

        # When transform is set as 'RI'/'RA', wrap fields should not be expected
        # The default transformation is GR, which outputs the wrap fields
        if not isdefined(transmodel) or (isdefined(transmodel)
                                         and transmodel not in ['RI', 'RA']):
            outputs['warp_field'] = os.path.join(
                os.getcwd(), self.inputs.out_prefix + 'Warp.nii.gz')
            outputs['inverse_warp_field'] = os.path.join(
                os.getcwd(), self.inputs.out_prefix + 'InverseWarp.nii.gz')

        outputs['affine_transformation'] = os.path.join(
            os.getcwd(), self.inputs.out_prefix + 'Affine.txt')
        outputs['input_file'] = os.path.join(
            os.getcwd(), self.inputs.out_prefix + 'repaired.nii.gz')
        outputs['output_file'] = os.path.join(
            os.getcwd(), self.inputs.out_prefix + 'deformed.nii.gz')

        return outputs


# How do we make a pass through so that GenWarpFields is just an alias for  antsIntroduction ?


class GenWarpFields(antsIntroduction):
    pass


class buildtemplateparallelInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        4,
        argstr='-d %d',
        usedefault=True,
        desc='image dimension (2, 3 or 4)',
        position=1)
    out_prefix = traits.Str(
        'antsTMPL_',
        argstr='-o %s',
        usedefault=True,
        desc=('Prefix that is prepended to all output '
              'files (default = antsTMPL_)'))
    in_files = traits.List(
        File(exists=True),
        mandatory=True,
        desc='list of images to generate template from',
        argstr='%s',
        position=-1)
    parallelization = traits.Enum(
        0,
        1,
        2,
        argstr='-c %d',
        usedefault=True,
        desc=('control for parallel processing (0 = '
              'serial, 1 = use PBS, 2 = use PEXEC, 3 = '
              'use Apple XGrid'))
    gradient_step_size = traits.Float(
        argstr='-g %f',
        desc=('smaller magnitude results in '
              'more cautious steps (default = '
              '.25)'))
    iteration_limit = traits.Int(
        4,
        argstr='-i %d',
        usedefault=True,
        desc='iterations of template construction')
    num_cores = traits.Int(
        argstr='-j %d',
        requires=['parallelization'],
        desc=('Requires parallelization = 2 (PEXEC). '
              'Sets number of cpu cores to use'))
    max_iterations = traits.List(
        traits.Int,
        argstr='-m %s',
        sep='x',
        desc=('maximum number of iterations (must be '
              'list of integers in the form [J,K,L...]: '
              'J = coarsest resolution iterations, K = '
              'middle resolution interations, L = fine '
              'resolution iterations'))
    bias_field_correction = traits.Bool(
        argstr='-n 1',
        desc=('Applies bias field correction to moving '
              'image'))
    rigid_body_registration = traits.Bool(
        argstr='-r 1',
        desc=('registers inputs before creating template '
              '(useful if no initial template available)'))
    similarity_metric = traits.Enum(
        'PR',
        'CC',
        'MI',
        'MSQ',
        argstr='-s %s',
        desc=('Type of similartiy metric used for registration '
              '(CC = cross correlation, MI = mutual information, '
              'PR = probability mapping, MSQ = mean square difference)'))
    transformation_model = traits.Enum(
        'GR',
        'EL',
        'SY',
        'S2',
        'EX',
        'DD',
        argstr='-t %s',
        usedefault=True,
        desc=('Type of transofmration model used for registration '
              '(EL = elastic transformation model, SY = SyN with time, '
              'arbitrary number of time points, S2 =  SyN with time '
              'optimized for 2 time points, GR = greedy SyN, EX = '
              'exponential, DD = diffeomorphic demons style exponential '
              'mapping'))
    use_first_as_target = traits.Bool(
        desc=('uses first volume as target of '
              'all inputs. When not used, an '
              'unbiased average image is used '
              'to start.'))


class buildtemplateparallelOutputSpec(TraitedSpec):
    final_template_file = File(exists=True, desc='final ANTS template')
    template_files = OutputMultiPath(
        File(exists=True), desc='Templates from different stages of iteration')
    subject_outfiles = OutputMultiPath(
        File(exists=True),
        desc=('Outputs for each input image. Includes warp '
              'field, inverse warp, Affine, original image '
              '(repaired) and warped image (deformed)'))


class buildtemplateparallel(ANTSCommand):
    """Generate a optimal average template

    .. warning::

      This can take a VERY long time to complete

    Examples
    --------

    >>> from nipype.interfaces.ants.legacy import buildtemplateparallel
    >>> tmpl = buildtemplateparallel()
    >>> tmpl.inputs.in_files = ['T1.nii', 'structural.nii']
    >>> tmpl.inputs.max_iterations = [30, 90, 20]
    >>> tmpl.cmdline
    'buildtemplateparallel.sh -d 3 -i 4 -m 30x90x20 -o antsTMPL_ -c 0 -t GR T1.nii structural.nii'

    """

    _cmd = 'buildtemplateparallel.sh'
    input_spec = buildtemplateparallelInputSpec
    output_spec = buildtemplateparallelOutputSpec

    def _format_arg(self, opt, spec, val):
        if opt == 'num_cores':
            if self.inputs.parallelization == 2:
                return '-j ' + str(val)
            else:
                return ''
        if opt == 'in_files':
            if self.inputs.use_first_as_target:
                start = '-z '
            else:
                start = ''
            return start + ' '.join(name for name in val)
        return super(buildtemplateparallel, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['template_files'] = []
        for i in range(len(glob(os.path.realpath('*iteration*')))):
            temp = os.path.realpath('%s_iteration_%d/%stemplate.nii.gz' %
                                    (self.inputs.transformation_model, i,
                                     self.inputs.out_prefix))
            os.rename(
                temp,
                os.path.realpath('%s_iteration_%d/%stemplate_i%d.nii.gz' %
                                 (self.inputs.transformation_model, i,
                                  self.inputs.out_prefix, i)))
            file_ = ('%s_iteration_%d/%stemplate_i%d.nii.gz' %
                     (self.inputs.transformation_model, i,
                      self.inputs.out_prefix, i))

            outputs['template_files'].append(os.path.realpath(file_))
            outputs['final_template_file'] = \
                os.path.realpath('%stemplate.nii.gz' %
                                 self.inputs.out_prefix)
        outputs['subject_outfiles'] = []
        for filename in self.inputs.in_files:
            _, base, _ = split_filename(filename)
            temp = glob(
                os.path.realpath('%s%s*' % (self.inputs.out_prefix, base)))
            for file_ in temp:
                outputs['subject_outfiles'].append(file_)
        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import SampleToSurface


def test_SampleToSurface_inputs():
    input_map = dict(
        apply_rot=dict(argstr='--rot %.3f %.3f %.3f', ),
        apply_trans=dict(argstr='--trans %.3f %.3f %.3f', ),
        args=dict(argstr='%s', ),
        cortex_mask=dict(
            argstr='--cortex',
            xor=['mask_label'],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fix_tk_reg=dict(argstr='--fixtkreg', ),
        float2int_method=dict(argstr='--float2int %s', ),
        frame=dict(argstr='--frame %d', ),
        hemi=dict(
            argstr='--hemi %s',
            mandatory=True,
        ),
        hits_file=dict(argstr='--srchit %s', ),
        hits_type=dict(argstr='--srchit_type', ),
        ico_order=dict(
            argstr='--icoorder %d',
            requires=['target_subject'],
        ),
        interp_method=dict(argstr='--interp %s', ),
        mask_label=dict(
            argstr='--mask %s',
            extensions=None,
            xor=['cortex_mask'],
        ),
        mni152reg=dict(
            argstr='--mni152reg',
            mandatory=True,
            xor=['reg_file', 'reg_header', 'mni152reg'],
        ),
        no_reshape=dict(
            argstr='--noreshape',
            xor=['reshape'],
        ),
        out_file=dict(
            argstr='--o %s',
            extensions=None,
            genfile=True,
        ),
        out_type=dict(argstr='--out_type %s', ),
        override_reg_subj=dict(
            argstr='--srcsubject %s',
            requires=['subject_id'],
        ),
        projection_stem=dict(
            mandatory=True,
            xor=['sampling_method'],
        ),
        reference_file=dict(
            argstr='--ref %s',
            extensions=None,
        ),
        reg_file=dict(
            argstr='--reg %s',
            extensions=None,
            mandatory=True,
            xor=['reg_file', 'reg_header', 'mni152reg'],
        ),
        reg_header=dict(
            argstr='--regheader %s',
            mandatory=True,
            requires=['subject_id'],
            xor=['reg_file', 'reg_header', 'mni152reg'],
        ),
        reshape=dict(
            argstr='--reshape',
            xor=['no_reshape'],
        ),
        reshape_slices=dict(argstr='--rf %d', ),
        sampling_method=dict(
            argstr='%s',
            mandatory=True,
            requires=['sampling_range', 'sampling_units'],
            xor=['projection_stem'],
        ),
        sampling_range=dict(),
        sampling_units=dict(),
        scale_input=dict(argstr='--scale %.3f', ),
        smooth_surf=dict(argstr='--surf-fwhm %.3f', ),
        smooth_vol=dict(argstr='--fwhm %.3f', ),
        source_file=dict(
            argstr='--mov %s',
            extensions=None,
            mandatory=True,
        ),
        subject_id=dict(),
        subjects_dir=dict(),
        surf_reg=dict(
            argstr='--surfreg %s',
            requires=['target_subject'],
        ),
        surface=dict(argstr='--surf %s', ),
        target_subject=dict(argstr='--trgsubject %s', ),
        vox_file=dict(argstr='--nvox %s', ),
    )
    inputs = SampleToSurface.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SampleToSurface_outputs():
    output_map = dict(
        hits_file=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        vox_file=dict(extensions=None, ),
    )
    outputs = SampleToSurface.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..svm import SVMTest


def test_SVMTest_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        classout=dict(argstr='-classout', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-testvol %s',
            extensions=None,
            mandatory=True,
        ),
        model=dict(
            argstr='-model %s',
            mandatory=True,
        ),
        multiclass=dict(argstr='-multiclass %s', ),
        nodetrend=dict(argstr='-nodetrend', ),
        nopredcensord=dict(argstr='-nopredcensord', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        options=dict(argstr='%s', ),
        out_file=dict(
            argstr='-predictions %s',
            extensions=None,
            name_template='%s_predictions',
        ),
        outputtype=dict(),
        testlabels=dict(
            argstr='-testlabels %s',
            extensions=None,
        ),
    )
    inputs = SVMTest.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SVMTest_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = SVMTest.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from future import standard_library
standard_library.install_aliases()
from builtins import open
import http.client
import inspect
import simplejson
import os
from subprocess import Popen, PIPE

import nipype


def is_git_repo():
    """Does the current nipype module have a git folder
    """
    sourcepath = os.path.realpath(
        os.path.join(os.path.dirname(nipype.__file__), os.path.pardir))
    gitpathgit = os.path.join(sourcepath, '.git')
    if os.path.exists(gitpathgit):
        return True
    else:
        return False


def get_local_branch():
    """Determine current branch
    """
    if is_git_repo():
        o, _ = Popen(
            'git branch | grep "\* "',
            shell=True,
            stdout=PIPE,
            cwd=os.path.dirname(nipype.__file__)).communicate()
        return o.strip()[2:]
    else:
        return None


def get_remote_branch():
    """Get remote branch for current branch
    """

    pass


def create_hash_map():
    """Create a hash map for all objects
    """

    hashmap = {}
    from base64 import encodestring as base64
    import pwd
    login_name = pwd.getpwuid(os.geteuid())[0]
    conn = http.client.HTTPSConnection("api.github.com")
    conn.request(
        "GET",
        "/repos/nipy/nipype",
        headers={
            'Authorization': 'Basic %s' % base64(login_name)
        })
    try:
        conn.request("GET", "/repos/nipy/nipype/git/trees/master?recursive=1")
    except:
        pass
    else:
        r1 = conn.getresponse()
        if r1.reason != 'OK':
            raise Exception('HTTP Response  %s:%s' % (r1.status, r1.reason))
        payload = simplejson.loads(r1.read())
        for infodict in payload['tree']:
            if infodict['type'] == "blob":
                hashmap[infodict['sha']] = infodict['path']
    return hashmap


def get_repo_url(force_github=False):
    """Returns github url or local url

    Returns
    -------
    URI: str
       filesystem path or github repo url
    """
    sourcepath = os.path.realpath(
        os.path.join(os.path.dirname(nipype.__file__), os.path.pardir))
    gitpathgit = os.path.join(sourcepath, '.git')
    if not os.path.exists(gitpathgit) and not force_github:
        uri = 'file://%s' % sourcepath
    else:
        uri = 'http://github.com/nipy/nipype/blob/master'
    return uri


def get_file_url(object):
    """Returns local or remote url for an object
    """
    filename = inspect.getsourcefile(object)
    lines = inspect.getsourcelines(object)
    uri = 'file://%s#L%d' % (filename, lines[1])
    if is_git_repo():
        info = nipype.get_info()
        shortfile = os.path.join('nipype', filename.split('nipype/')[-1])
        uri = 'http://github.com/nipy/nipype/tree/%s/%s#L%d' % \
            (info['commit_hash'],
             shortfile, lines[1])
    return uri



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..classify import BRAINSPosteriorToContinuousClass


def test_BRAINSPosteriorToContinuousClass_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputBasalGmVolume=dict(
            argstr='--inputBasalGmVolume %s',
            extensions=None,
        ),
        inputCrblGmVolume=dict(
            argstr='--inputCrblGmVolume %s',
            extensions=None,
        ),
        inputCrblWmVolume=dict(
            argstr='--inputCrblWmVolume %s',
            extensions=None,
        ),
        inputCsfVolume=dict(
            argstr='--inputCsfVolume %s',
            extensions=None,
        ),
        inputSurfaceGmVolume=dict(
            argstr='--inputSurfaceGmVolume %s',
            extensions=None,
        ),
        inputVbVolume=dict(
            argstr='--inputVbVolume %s',
            extensions=None,
        ),
        inputWhiteVolume=dict(
            argstr='--inputWhiteVolume %s',
            extensions=None,
        ),
        outputVolume=dict(
            argstr='--outputVolume %s',
            hash_files=False,
        ),
    )
    inputs = BRAINSPosteriorToContinuousClass.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSPosteriorToContinuousClass_outputs():
    output_map = dict(outputVolume=dict(extensions=None, ), )
    outputs = BRAINSPosteriorToContinuousClass.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..regutils import RegAverage


def test_RegAverage_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        avg_files=dict(
            argstr='-avg %s',
            position=1,
            sep=' ',
            xor=[
                'avg_lts_files', 'avg_ref_file', 'demean1_ref_file',
                'demean2_ref_file', 'demean3_ref_file', 'warp_files'
            ],
        ),
        avg_lts_files=dict(
            argstr='-avg_lts %s',
            position=1,
            sep=' ',
            xor=[
                'avg_files', 'avg_ref_file', 'demean1_ref_file',
                'demean2_ref_file', 'demean3_ref_file', 'warp_files'
            ],
        ),
        avg_ref_file=dict(
            argstr='-avg_tran %s',
            extensions=None,
            position=1,
            requires=['warp_files'],
            xor=[
                'avg_files', 'avg_lts_files', 'demean1_ref_file',
                'demean2_ref_file', 'demean3_ref_file'
            ],
        ),
        demean1_ref_file=dict(
            argstr='-demean1 %s',
            extensions=None,
            position=1,
            requires=['warp_files'],
            xor=[
                'avg_files', 'avg_lts_files', 'avg_ref_file',
                'demean2_ref_file', 'demean3_ref_file'
            ],
        ),
        demean2_ref_file=dict(
            argstr='-demean2 %s',
            extensions=None,
            position=1,
            requires=['warp_files'],
            xor=[
                'avg_files', 'avg_lts_files', 'avg_ref_file',
                'demean1_ref_file', 'demean3_ref_file'
            ],
        ),
        demean3_ref_file=dict(
            argstr='-demean3 %s',
            extensions=None,
            position=1,
            requires=['warp_files'],
            xor=[
                'avg_files', 'avg_lts_files', 'avg_ref_file',
                'demean1_ref_file', 'demean2_ref_file'
            ],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        omp_core_val=dict(
            argstr='-omp %i',
            usedefault=True,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=0,
        ),
        warp_files=dict(
            argstr='%s',
            position=-1,
            sep=' ',
            xor=['avg_files', 'avg_lts_files'],
        ),
    )
    inputs = RegAverage.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RegAverage_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = RegAverage.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..misc import ModifyAffine


def test_ModifyAffine_inputs():
    input_map = dict(
        transformation_matrix=dict(usedefault=True, ),
        volumes=dict(mandatory=True, ),
    )
    inputs = ModifyAffine.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ModifyAffine_outputs():
    output_map = dict(transformed_volumes=dict(), )
    outputs = ModifyAffine.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from future import standard_library
from ..nipype2boutiques import generate_boutiques_descriptor
from nipype.testing import example_data
import json
standard_library.install_aliases()


def test_generate():
    ignored_inputs = [
        "args",
        "environ",
        "output_type"
    ]
    desc = generate_boutiques_descriptor(module='nipype.interfaces.fsl',
                                         interface_name='FLIRT',
                                         container_image=('mcin/'
                                                          'docker-fsl:latest'),
                                         container_index='index.docker.io',
                                         container_type='docker',
                                         verbose=False,
                                         save=False,
                                         ignore_inputs=ignored_inputs,
                                         author=("Oxford Centre for Functional"
                                                 " MRI of the Brain (FMRIB)"))

    with open(example_data('nipype2boutiques_example.json'), 'r') as desc_file:
        # Make sure that output descriptor matches the expected descriptor.
        output_desc = json.loads(desc)
        expected_desc = json.load(desc_file)
        assert (output_desc.get('name') ==
                expected_desc.get('name'))
        assert (output_desc.get('author') ==
                expected_desc.get('author'))
        assert (output_desc.get('command-line') ==
                expected_desc.get('command-line'))
        assert (output_desc.get('description') ==
                expected_desc.get('description'))
        assert (len(output_desc.get('inputs')) ==
                len(expected_desc.get('inputs')))
        assert(len(output_desc.get('output-files')) ==
               len(expected_desc.get('output-files')))
        assert (output_desc.get('container-image').get('image') ==
                expected_desc.get('container-image').get('image'))



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import OneDToolPy


def test_OneDToolPy_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        censor_motion=dict(argstr='-censor_motion %f %s', ),
        censor_prev_TR=dict(argstr='-censor_prev_TR', ),
        demean=dict(argstr='-demean', ),
        derivative=dict(argstr='-derivative', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-infile %s',
            extensions=None,
            mandatory=True,
        ),
        out_file=dict(
            argstr='-write %s',
            extensions=None,
            xor=['show_cormat_warnings'],
        ),
        outputtype=dict(),
        py27_path=dict(usedefault=True, ),
        set_nruns=dict(argstr='-set_nruns %d', ),
        show_censor_count=dict(argstr='-show_censor_count', ),
        show_cormat_warnings=dict(
            argstr='-show_cormat_warnings |& tee %s',
            extensions=None,
            position=-1,
            xor=['out_file'],
        ),
        show_indices_interest=dict(argstr='-show_indices_interest', ),
        show_trs_run=dict(argstr='-show_trs_run %d', ),
        show_trs_uncensored=dict(argstr='-show_trs_uncensored %s', ),
    )
    inputs = OneDToolPy.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_OneDToolPy_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = OneDToolPy.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""ANTS Apply Transforms interface
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import os

from ..base import TraitedSpec, File, traits, InputMultiPath
from .base import ANTSCommand, ANTSCommandInputSpec


class AverageAffineTransformInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        mandatory=True,
        position=0,
        desc='image dimension (2 or 3)')
    output_affine_transform = File(
        argstr='%s',
        mandatory=True,
        position=1,
        desc='Outputfname.txt: the name of the resulting transform.')
    transforms = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        position=3,
        desc='transforms to average')


class AverageAffineTransformOutputSpec(TraitedSpec):
    affine_transform = File(exists=True, desc='average transform file')


class AverageAffineTransform(ANTSCommand):
    """
    Examples
    --------
    >>> from nipype.interfaces.ants import AverageAffineTransform
    >>> avg = AverageAffineTransform()
    >>> avg.inputs.dimension = 3
    >>> avg.inputs.transforms = ['trans.mat', 'func_to_struct.mat']
    >>> avg.inputs.output_affine_transform = 'MYtemplatewarp.mat'
    >>> avg.cmdline
    'AverageAffineTransform 3 MYtemplatewarp.mat trans.mat func_to_struct.mat'
    """
    _cmd = 'AverageAffineTransform'
    input_spec = AverageAffineTransformInputSpec
    output_spec = AverageAffineTransformOutputSpec

    def _format_arg(self, opt, spec, val):
        return super(AverageAffineTransform, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['affine_transform'] = os.path.abspath(
            self.inputs.output_affine_transform)
        return outputs


class AverageImagesInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        mandatory=True,
        position=0,
        desc='image dimension (2 or 3)')
    output_average_image = File(
        "average.nii",
        argstr='%s',
        position=1,
        usedefault=True,
        hash_files=False,
        desc='the name of the resulting image.')
    normalize = traits.Bool(
        argstr="%d",
        mandatory=True,
        position=2,
        desc='Normalize: if true, the 2nd image is divided by its mean. '
        'This will select the largest image to average into.')
    images = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        position=3,
        desc=
        'image to apply transformation to (generally a coregistered functional)'
    )


class AverageImagesOutputSpec(TraitedSpec):
    output_average_image = File(exists=True, desc='average image file')


class AverageImages(ANTSCommand):
    """
    Examples
    --------
    >>> from nipype.interfaces.ants import AverageImages
    >>> avg = AverageImages()
    >>> avg.inputs.dimension = 3
    >>> avg.inputs.output_average_image = "average.nii.gz"
    >>> avg.inputs.normalize = True
    >>> avg.inputs.images = ['rc1s1.nii', 'rc1s1.nii']
    >>> avg.cmdline
    'AverageImages 3 average.nii.gz 1 rc1s1.nii rc1s1.nii'
    """
    _cmd = 'AverageImages'
    input_spec = AverageImagesInputSpec
    output_spec = AverageImagesOutputSpec

    def _format_arg(self, opt, spec, val):
        return super(AverageImages, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output_average_image'] = os.path.realpath(
            self.inputs.output_average_image)
        return outputs


class MultiplyImagesInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        mandatory=True,
        position=0,
        desc='image dimension (2 or 3)')
    first_input = File(
        argstr='%s', exists=True, mandatory=True, position=1, desc='image 1')
    second_input = traits.Either(
        File(exists=True),
        traits.Float,
        argstr='%s',
        mandatory=True,
        position=2,
        desc='image 2 or multiplication weight')
    output_product_image = File(
        argstr='%s',
        mandatory=True,
        position=3,
        desc='Outputfname.nii.gz: the name of the resulting image.')


class MultiplyImagesOutputSpec(TraitedSpec):
    output_product_image = File(exists=True, desc='average image file')


class MultiplyImages(ANTSCommand):
    """
    Examples
    --------
    >>> from nipype.interfaces.ants import MultiplyImages
    >>> test = MultiplyImages()
    >>> test.inputs.dimension = 3
    >>> test.inputs.first_input = 'moving2.nii'
    >>> test.inputs.second_input = 0.25
    >>> test.inputs.output_product_image = "out.nii"
    >>> test.cmdline
    'MultiplyImages 3 moving2.nii 0.25 out.nii'
    """
    _cmd = 'MultiplyImages'
    input_spec = MultiplyImagesInputSpec
    output_spec = MultiplyImagesOutputSpec

    def _format_arg(self, opt, spec, val):
        return super(MultiplyImages, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['output_product_image'] = os.path.abspath(
            self.inputs.output_product_image)
        return outputs


class CreateJacobianDeterminantImageInputSpec(ANTSCommandInputSpec):
    imageDimension = traits.Enum(
        3,
        2,
        argstr='%d',
        mandatory=True,
        position=0,
        desc='image dimension (2 or 3)')
    deformationField = File(
        argstr='%s',
        exists=True,
        mandatory=True,
        position=1,
        desc='deformation transformation file')
    outputImage = File(
        argstr='%s', mandatory=True, position=2, desc='output filename')
    doLogJacobian = traits.Enum(
        0, 1, argstr='%d', position=3, desc='return the log jacobian')
    useGeometric = traits.Enum(
        0, 1, argstr='%d', position=4, desc='return the geometric jacobian')


class CreateJacobianDeterminantImageOutputSpec(TraitedSpec):
    jacobian_image = File(exists=True, desc='jacobian image')


class CreateJacobianDeterminantImage(ANTSCommand):
    """
    Examples
    --------
    >>> from nipype.interfaces.ants import CreateJacobianDeterminantImage
    >>> jacobian = CreateJacobianDeterminantImage()
    >>> jacobian.inputs.imageDimension = 3
    >>> jacobian.inputs.deformationField = 'ants_Warp.nii.gz'
    >>> jacobian.inputs.outputImage = 'out_name.nii.gz'
    >>> jacobian.cmdline
    'CreateJacobianDeterminantImage 3 ants_Warp.nii.gz out_name.nii.gz'
    """

    _cmd = 'CreateJacobianDeterminantImage'
    input_spec = CreateJacobianDeterminantImageInputSpec
    output_spec = CreateJacobianDeterminantImageOutputSpec

    def _format_arg(self, opt, spec, val):
        return super(CreateJacobianDeterminantImage, self)._format_arg(
            opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['jacobian_image'] = os.path.abspath(self.inputs.outputImage)
        return outputs


class AffineInitializerInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3, 2, usedefault=True, position=0, argstr='%s', desc='dimension')
    fixed_image = File(
        exists=True,
        mandatory=True,
        position=1,
        argstr='%s',
        desc='reference image')
    moving_image = File(
        exists=True,
        mandatory=True,
        position=2,
        argstr='%s',
        desc='moving image')
    out_file = File(
        'transform.mat',
        usedefault=True,
        position=3,
        argstr='%s',
        desc='output transform file')
    # Defaults in antsBrainExtraction.sh -> 15 0.1 0 10
    search_factor = traits.Float(
        15.0,
        usedefault=True,
        position=4,
        argstr='%f',
        desc='increments (degrees) for affine search')
    radian_fraction = traits.Range(
        0.0,
        1.0,
        value=0.1,
        usedefault=True,
        position=5,
        argstr='%f',
        desc='search this arc +/- principal axes')
    principal_axes = traits.Bool(
        False,
        usedefault=True,
        position=6,
        argstr='%d',
        desc=
        'whether the rotation is searched around an initial principal axis alignment.'
    )
    local_search = traits.Int(
        10,
        usedefault=True,
        position=7,
        argstr='%d',
        desc=
        ' determines if a local optimization is run at each search point for the set '
        'number of iterations')


class AffineInitializerOutputSpec(TraitedSpec):
    out_file = File(desc='output transform file')


class AffineInitializer(ANTSCommand):
    """
    Initialize an affine transform (as in antsBrainExtraction.sh)

    >>> from nipype.interfaces.ants import AffineInitializer
    >>> init = AffineInitializer()
    >>> init.inputs.fixed_image = 'fixed1.nii'
    >>> init.inputs.moving_image = 'moving1.nii'
    >>> init.cmdline
    'antsAffineInitializer 3 fixed1.nii moving1.nii transform.mat 15.000000 0.100000 0 10'

    """
    _cmd = 'antsAffineInitializer'
    input_spec = AffineInitializerInputSpec
    output_spec = AffineInitializerOutputSpec

    def _list_outputs(self):
        return {'out_file': os.path.abspath(self.inputs.out_file)}


class ComposeMultiTransformInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        usedefault=True,
        position=0,
        desc='image dimension (2 or 3)')
    output_transform = File(
        argstr='%s',
        position=1,
        name_source=['transforms'],
        name_template='%s_composed',
        keep_extension=True,
        desc='the name of the resulting transform.')
    reference_image = File(
        argstr='%s',
        position=2,
        desc='Reference image (only necessary when output is warpfield)')
    transforms = InputMultiPath(
        File(exists=True),
        argstr='%s',
        mandatory=True,
        position=3,
        desc='transforms to average')


class ComposeMultiTransformOutputSpec(TraitedSpec):
    output_transform = File(exists=True, desc='Composed transform file')


class ComposeMultiTransform(ANTSCommand):
    """
    Take a set of transformations and convert them to a single transformation matrix/warpfield.

    Examples
    --------
    >>> from nipype.interfaces.ants import ComposeMultiTransform
    >>> compose_transform = ComposeMultiTransform()
    >>> compose_transform.inputs.dimension = 3
    >>> compose_transform.inputs.transforms = ['struct_to_template.mat', 'func_to_struct.mat']
    >>> compose_transform.cmdline
    'ComposeMultiTransform 3 struct_to_template_composed.mat struct_to_template.mat func_to_struct.mat'

    """
    _cmd = 'ComposeMultiTransform'
    input_spec = ComposeMultiTransformInputSpec
    output_spec = ComposeMultiTransformOutputSpec


class LabelGeometryInputSpec(ANTSCommandInputSpec):
    dimension = traits.Enum(
        3,
        2,
        argstr='%d',
        usedefault=True,
        position=0,
        desc='image dimension (2 or 3)')
    label_image = File(
        argstr='%s',
        position=1,
        mandatory=True,
        desc='label image to use for extracting geometry measures')
    intensity_image = File(
        value='[]',
        exists=True,
        argstr='%s',
        mandatory=True,
        usedefault=True,
        position=2,
        desc='Intensity image to extract values from. '
             'This is an optional input')
    output_file = traits.Str(
        name_source=['label_image'],
        name_template='%s.csv',
        argstr='%s',
        position=3,
        desc='name of output file')


class LabelGeometryOutputSpec(TraitedSpec):
    output_file = File(exists=True, desc='CSV file of geometry measures')


class LabelGeometry(ANTSCommand):
    """
    Extracts geometry measures using a label file and an optional image file

    Examples
    --------
    >>> from nipype.interfaces.ants import LabelGeometry
    >>> label_extract = LabelGeometry()
    >>> label_extract.inputs.dimension = 3
    >>> label_extract.inputs.label_image = 'atlas.nii.gz'
    >>> label_extract.cmdline
    'LabelGeometryMeasures 3 atlas.nii.gz [] atlas.csv'

    >>> label_extract.inputs.intensity_image = 'ants_Warp.nii.gz'
    >>> label_extract.cmdline
    'LabelGeometryMeasures 3 atlas.nii.gz ants_Warp.nii.gz atlas.csv'

    """
    _cmd = 'LabelGeometryMeasures'
    input_spec = LabelGeometryInputSpec
    output_spec = LabelGeometryOutputSpec



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..convert import Camino2Trackvis


def test_Camino2Trackvis_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        data_dims=dict(
            argstr='-d %s',
            mandatory=True,
            position=4,
            sep=',',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        min_length=dict(
            argstr='-l %d',
            position=3,
            units='mm',
        ),
        nifti_file=dict(
            argstr='--nifti %s',
            extensions=None,
            position=7,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
            position=2,
        ),
        voxel_dims=dict(
            argstr='-x %s',
            mandatory=True,
            position=5,
            sep=',',
        ),
        voxel_order=dict(
            argstr='--voxel-order %s',
            extensions=None,
            mandatory=True,
            position=6,
        ),
    )
    inputs = Camino2Trackvis.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Camino2Trackvis_outputs():
    output_map = dict(trackvis=dict(extensions=None, ), )
    outputs = Camino2Trackvis.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import MRIsExpand


def test_MRIsExpand_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        distance=dict(
            argstr='%g',
            mandatory=True,
            position=-2,
        ),
        dt=dict(argstr='-T %g', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        nsurfaces=dict(argstr='-N %d', ),
        out_name=dict(
            argstr='%s',
            position=-1,
            usedefault=True,
        ),
        pial=dict(
            argstr='-pial %s',
            copyfile=False,
        ),
        smooth_averages=dict(argstr='-A %d', ),
        sphere=dict(
            copyfile=False,
            usedefault=True,
        ),
        spring=dict(argstr='-S %g', ),
        subjects_dir=dict(),
        thickness=dict(argstr='-thickness', ),
        thickness_name=dict(
            argstr='-thickness_name %s',
            copyfile=False,
        ),
        write_iterations=dict(argstr='-W %d', ),
    )
    inputs = MRIsExpand.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MRIsExpand_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = MRIsExpand.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import division
import numpy as np
from nipype.algorithms.icc import ICC_rep_anova


def test_ICC_rep_anova():
    # see table 2 in P. E. Shrout & Joseph L. Fleiss (1979). "Intraclass
    # Correlations: Uses in Assessing Rater Reliability". Psychological
    # Bulletin 86 (2): 420-428
    Y = np.array([[9, 2, 5, 8], [6, 1, 3, 2], [8, 4, 6, 8], [7, 1, 2, 6],
                  [10, 5, 6, 9], [6, 2, 4, 7]])

    icc, r_var, e_var, _, dfc, dfe = ICC_rep_anova(Y)
    # see table 4
    assert round(icc, 2) == 0.71
    assert dfc == 3
    assert dfe == 15
    assert np.isclose(r_var / (r_var + e_var), icc)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The ASL module of niftyfit, which wraps the fitting methods in NiftyFit.
"""

from ..base import File, TraitedSpec, traits, CommandLineInputSpec
from .base import NiftyFitCommand
from ..niftyreg.base import get_custom_path


class FitAslInputSpec(CommandLineInputSpec):
    """ Input Spec for FitAsl. """
    desc = 'Filename of the 4D ASL (control/label) source image (mandatory).'
    source_file = File(
        position=1,
        exists=True,
        argstr='-source %s',
        mandatory=True,
        desc=desc)
    pasl = traits.Bool(desc='Fit PASL ASL data [default]', argstr='-pasl')
    pcasl = traits.Bool(desc='Fit PCASL ASL data', argstr='-pcasl')

    # *** Output options:
    desc = 'Filename of the Cerebral Blood Flow map (in ml/100g/min).'
    cbf_file = File(
        name_source=['source_file'],
        name_template='%s_cbf.nii.gz',
        argstr='-cbf %s',
        desc=desc)
    error_file = File(
        name_source=['source_file'],
        name_template='%s_error.nii.gz',
        argstr='-error %s',
        desc='Filename of the CBF error map.')
    syn_file = File(
        name_source=['source_file'],
        name_template='%s_syn.nii.gz',
        argstr='-syn %s',
        desc='Filename of the synthetic ASL data.')

    # *** Input options (see also fit_qt1 for generic T1 fitting):
    desc = 'Filename of the estimated input T1 map (in ms).'
    t1map = File(exists=True, argstr='-t1map %s', desc=desc)
    desc = 'Filename of the estimated input M0 map.'
    m0map = File(exists=True, argstr='-m0map %s', desc=desc)
    desc = 'Filename of the estimated input M0 map error.'
    m0mape = File(exists=True, argstr='-m0mape %s', desc=desc)
    desc = 'Filename of a [1,2,5]s Inversion Recovery volume (T1/M0 fitting \
carried out internally).'

    ir_volume = File(exists=True, argstr='-IRvolume %s', desc=desc)
    desc = 'Output of [1,2,5]s Inversion Recovery fitting.'
    ir_output = File(exists=True, argstr='-IRoutput %s', desc=desc)

    # *** Experimental options (Choose those suitable for the model!):
    mask = File(
        position=2,
        exists=True,
        desc='Filename of image mask.',
        argstr='-mask %s')
    t1_art_cmp = traits.Float(
        desc='T1 of arterial component [1650ms].', argstr='-T1a %f')
    desc = 'Single plasma/tissue partition coefficient [0.9ml/g].'
    plasma_coeff = traits.Float(desc=desc, argstr='-L %f')
    desc = 'Labelling efficiency [0.99 (pasl), 0.85 (pcasl)], ensure any \
background suppression pulses are included in -eff'

    eff = traits.Float(desc=desc, argstr='-eff %f')
    desc = 'Outlier rejection for multi CL volumes (enter z-score threshold \
(e.g. 2.5)) [off].'

    out = traits.Float(desc=desc, argstr='-out %f')

    # *** PCASL options (Choose those suitable for the model!):
    pld = traits.Float(desc='Post Labelling Delay [2000ms].', argstr='-PLD %f')
    ldd = traits.Float(desc='Labelling Duration [1800ms].', argstr='-LDD %f')
    desc = 'Difference in labelling delay per slice [0.0 ms/slice.'
    dpld = traits.Float(desc=desc, argstr='-dPLD %f')

    # *** PASL options (Choose those suitable for the model!):
    t_inv1 = traits.Float(
        desc='Saturation pulse time [800ms].', argstr='-Tinv1 %f')
    t_inv2 = traits.Float(desc='Inversion time [2000ms].', argstr='-Tinv2 %f')
    desc = 'Difference in inversion time per slice [0ms/slice].'
    dt_inv2 = traits.Float(desc=desc, argstr='-dTinv2 %f')

    # *** Other experimental assumptions:

    # Not programmed yet
    # desc = 'Slope and intercept for Arterial Transit Time.'
    # ATT = traits.Float(desc=desc, argstr='-ATT %f')

    gm_t1 = traits.Float(desc='T1 of GM [1150ms].', argstr='-gmT1 %f')
    gm_plasma = traits.Float(
        desc='Plasma/GM water partition [0.95ml/g].', argstr='-gmL %f')
    gm_ttt = traits.Float(desc='Time to GM [ATT+0ms].', argstr='-gmTTT %f')
    wm_t1 = traits.Float(desc='T1 of WM [800ms].', argstr='-wmT1 %f')
    wm_plasma = traits.Float(
        desc='Plasma/WM water partition [0.82ml/g].', argstr='-wmL %f')
    wm_ttt = traits.Float(desc='Time to WM [ATT+0ms].', argstr='-wmTTT %f')

    # *** Segmentation options:
    desc = 'Filename of the 4D segmentation (in ASL space) for L/T1 \
estimation and PV correction {WM,GM,CSF}.'

    seg = File(exists=True, argstr='-seg %s', desc=desc)
    desc = 'Use sigmoid to estimate L from T1: L(T1|gmL,wmL) [Off].'
    sig = traits.Bool(desc=desc, argstr='-sig')
    desc = 'Simple PV correction (CBF=vg*CBFg + vw*CBFw, with CBFw=f*CBFg) \
[0.25].'

    pv0 = traits.Int(desc=desc, argstr='-pv0 %d')
    pv2 = traits.Int(desc='In plane PV kernel size [3x3].', argstr='-pv2 %d')
    pv3 = traits.Tuple(
        traits.Int,
        traits.Int,
        traits.Int,
        desc='3D kernel size [3x3x1].',
        argstr='-pv3 %d %d %d')
    desc = 'Multiply CBF by this value (e.g. if CL are mislabelled use -1.0).'
    mul = traits.Float(desc=desc, argstr='-mul %f')
    mulgm = traits.Bool(
        desc='Multiply CBF by segmentation [Off].', argstr='-sig')
    desc = 'Set PV threshold for switching off LSQR [O.05].'
    pv_threshold = traits.Bool(desc=desc, argstr='-pvthreshold')
    segstyle = traits.Bool(
        desc='Set CBF as [gm,wm] not [wm,gm].', argstr='-segstyle')


class FitAslOutputSpec(TraitedSpec):
    """ Output Spec for FitAsl. """
    desc = 'Filename of the Cerebral Blood Flow map (in ml/100g/min).'
    cbf_file = File(exists=True, desc=desc)
    desc = 'Filename of the CBF error map.'
    error_file = File(exists=True, desc=desc)
    desc = 'Filename of the synthetic ASL data.'
    syn_file = File(exists=True, desc=desc)


class FitAsl(NiftyFitCommand):
    """Interface for executable fit_asl from Niftyfit platform.

    Use NiftyFit to perform ASL fitting.

    ASL fitting routines (following EU Cost Action White Paper recommendations)
    Fits Cerebral Blood Flow maps in the first instance.

    `Source code <https://cmiclab.cs.ucl.ac.uk/CMIC/NiftyFit-Release>`_

    Examples
    --------
    >>> from nipype.interfaces import niftyfit
    >>> node = niftyfit.FitAsl()
    >>> node.inputs.source_file = 'asl.nii.gz'
    >>> node.cmdline
    'fit_asl -source asl.nii.gz -cbf asl_cbf.nii.gz -error asl_error.nii.gz \
-syn asl_syn.nii.gz'

    """
    _cmd = get_custom_path('fit_asl', env_dir='NIFTYFITDIR')
    input_spec = FitAslInputSpec
    output_spec = FitAslOutputSpec
    _suffix = '_fit_asl'



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import warnings
import nibabel as nb

from .base import NipyBaseInterface, have_nipy
from ..base import (TraitedSpec, traits, BaseInterfaceInputSpec,
                    File, isdefined)


class SimilarityInputSpec(BaseInterfaceInputSpec):
    volume1 = File(exists=True, desc="3D volume", mandatory=True)
    volume2 = File(exists=True, desc="3D volume", mandatory=True)
    mask1 = File(exists=True, desc="3D volume")
    mask2 = File(exists=True, desc="3D volume")
    metric = traits.Either(
        traits.Enum('cc', 'cr', 'crl1', 'mi', 'nmi', 'slr'),
        traits.Callable(),
        desc="""str or callable
Cost-function for assessing image similarity. If a string,
one of 'cc': correlation coefficient, 'cr': correlation
ratio, 'crl1': L1-norm based correlation ratio, 'mi': mutual
information, 'nmi': normalized mutual information, 'slr':
supervised log-likelihood ratio. If a callable, it should
take a two-dimensional array representing the image joint
histogram as an input and return a float.""",
        usedefault=True)


class SimilarityOutputSpec(TraitedSpec):
    similarity = traits.Float(desc="Similarity between volume 1 and 2")


class Similarity(NipyBaseInterface):
    """Calculates similarity between two 3D volumes. Both volumes have to be in
    the same coordinate system, same space within that coordinate system and
    with the same voxel dimensions.

    .. deprecated:: 0.10.0
       Use :py:class:`nipype.algorithms.metrics.Similarity` instead.

    Example
    -------
    >>> from nipype.interfaces.nipy.utils import Similarity
    >>> similarity = Similarity()
    >>> similarity.inputs.volume1 = 'rc1s1.nii'
    >>> similarity.inputs.volume2 = 'rc1s2.nii'
    >>> similarity.inputs.mask1 = 'mask.nii'
    >>> similarity.inputs.mask2 = 'mask.nii'
    >>> similarity.inputs.metric = 'cr'
    >>> res = similarity.run() # doctest: +SKIP
    """

    input_spec = SimilarityInputSpec
    output_spec = SimilarityOutputSpec

    def __init__(self, **inputs):
        warnings.warn(("This interface is deprecated since 0.10.0."
                       " Please use nipype.algorithms.metrics.Similarity"),
                      DeprecationWarning)
        super(Similarity, self).__init__(**inputs)

    def _run_interface(self, runtime):
        from nipy.algorithms.registration.histogram_registration import HistogramRegistration
        from nipy.algorithms.registration.affine import Affine

        vol1_nii = nb.load(self.inputs.volume1)
        vol2_nii = nb.load(self.inputs.volume2)

        if isdefined(self.inputs.mask1):
            mask1 = nb.load(self.inputs.mask1).get_data() == 1
        else:
            mask1 = None

        if isdefined(self.inputs.mask2):
            mask2 = nb.load(self.inputs.mask2).get_data() == 1
        else:
            mask2 = None

        histreg = HistogramRegistration(
            from_img=vol1_nii,
            to_img=vol2_nii,
            similarity=self.inputs.metric,
            from_mask=mask1,
            to_mask=mask2)
        self._similarity = histreg.eval(Affine())

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['similarity'] = self._similarity
        return outputs



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The niftyfit module provides classes for interfacing with the `NiftyFit`_
command line tools.

Top-level namespace for niftyfit.
"""

from .asl import FitAsl
from .dwi import FitDwi, DwiTool
from .qt1 import FitQt1



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from .changequantification import IntensityDifferenceMetric
from .petstandarduptakevaluecomputation import PETStandardUptakeValueComputation



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from builtins import str
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from builtins import open

import os
from copy import deepcopy

import pytest
import pdb
from nipype.utils.filemanip import split_filename, ensure_list
from .. import preprocess as fsl
from nipype.interfaces.fsl import Info
from nipype.interfaces.base import File, TraitError, Undefined, isdefined
from nipype.interfaces.fsl import no_fsl


def fsl_name(obj, fname):
    """Create valid fsl name, including file extension for output type.
    """
    ext = Info.output_type_to_ext(obj.inputs.output_type)
    return fname + ext


@pytest.fixture()
def setup_infile(tmpdir):
    ext = Info.output_type_to_ext(Info.output_type())
    tmp_infile = tmpdir.join('foo' + ext)
    tmp_infile.open("w")
    return (tmp_infile.strpath, tmpdir.strpath)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_bet(setup_infile):
    tmp_infile, tp_dir = setup_infile
    better = fsl.BET()
    assert better.cmd == 'bet'

    # Test raising error with mandatory args absent
    with pytest.raises(ValueError):
        better.run()

    # Test generated outfile name
    better.inputs.in_file = tmp_infile
    outfile = fsl_name(better, 'foo_brain')
    outpath = os.path.join(os.getcwd(), outfile)
    realcmd = 'bet %s %s' % (tmp_infile, outpath)
    assert better.cmdline == realcmd
    # Test specified outfile name
    outfile = fsl_name(better, '/newdata/bar')
    better.inputs.out_file = outfile
    realcmd = 'bet %s %s' % (tmp_infile, outfile)
    assert better.cmdline == realcmd

    # infile foo.nii doesn't exist
    def func():
        better.run(in_file='foo2.nii', out_file='bar.nii')

    with pytest.raises(TraitError):
        func()

    # Our options and some test values for them
    # Should parallel the opt_map structure in the class for clarity
    opt_map = {
        'outline': ('-o', True),
        'mask': ('-m', True),
        'skull': ('-s', True),
        'no_output': ('-n', True),
        'frac': ('-f 0.40', 0.4),
        'vertical_gradient': ('-g 0.75', 0.75),
        'radius': ('-r 20', 20),
        'center': ('-c 54 75 80', [54, 75, 80]),
        'threshold': ('-t', True),
        'mesh': ('-e', True),
        'surfaces': ('-A', True)
        # 'verbose': ('-v', True),
        # 'flags': ('--i-made-this-up', '--i-made-this-up'),
    }
    # Currently we don't test -R, -S, -B, -Z, -F, -A or -A2

    # test each of our arguments
    better = fsl.BET()
    outfile = fsl_name(better, 'foo_brain')
    outpath = os.path.join(os.getcwd(), outfile)
    for name, settings in list(opt_map.items()):
        better = fsl.BET(**{name: settings[1]})
        # Add mandatory input
        better.inputs.in_file = tmp_infile
        realcmd = ' '.join([better.cmd, tmp_infile, outpath, settings[0]])
        assert better.cmdline == realcmd


# test fast


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fast(setup_infile):
    tmp_infile, tp_dir = setup_infile
    faster = fsl.FAST()
    faster.inputs.verbose = True
    fasted = fsl.FAST(in_files=tmp_infile, verbose=True)
    fasted2 = fsl.FAST(in_files=[tmp_infile, tmp_infile], verbose=True)

    assert faster.cmd == 'fast'
    assert faster.inputs.verbose
    assert faster.inputs.manual_seg == Undefined
    assert faster.inputs != fasted.inputs
    assert fasted.cmdline == 'fast -v -S 1 %s' % (tmp_infile)
    assert fasted2.cmdline == 'fast -v -S 2 %s %s' % (tmp_infile, tmp_infile)

    faster = fsl.FAST()
    faster.inputs.in_files = tmp_infile
    assert faster.cmdline == 'fast -S 1 %s' % (tmp_infile)
    faster.inputs.in_files = [tmp_infile, tmp_infile]
    assert faster.cmdline == 'fast -S 2 %s %s' % (tmp_infile, tmp_infile)

    # Our options and some test values for them
    # Should parallel the opt_map structure in the class for clarity
    opt_map = {
        'number_classes': ('-n 4', 4),
        'bias_iters': ('-I 5', 5),
        'bias_lowpass': ('-l 15', 15),
        'img_type': ('-t 2', 2),
        'init_seg_smooth': ('-f 0.035', 0.035),
        'segments': ('-g', True),
        'init_transform': ('-a %s' % (tmp_infile), '%s' % (tmp_infile)),
        'other_priors':
        ('-A %s %s %s' % (tmp_infile, tmp_infile, tmp_infile),
         (['%s' % (tmp_infile),
           '%s' % (tmp_infile),
           '%s' % (tmp_infile)])),
        'no_pve': ('--nopve', True),
        'output_biasfield': ('-b', True),
        'output_biascorrected': ('-B', True),
        'no_bias': ('-N', True),
        'out_basename': ('-o fasted', 'fasted'),
        'use_priors': ('-P', True),
        'segment_iters': ('-W 14', 14),
        'mixel_smooth': ('-R 0.25', 0.25),
        'iters_afterbias': ('-O 3', 3),
        'hyper': ('-H 0.15', 0.15),
        'verbose': ('-v', True),
        'manual_seg': ('-s %s' % (tmp_infile), '%s' % (tmp_infile)),
        'probability_maps': ('-p', True),
    }

    # test each of our arguments
    for name, settings in list(opt_map.items()):
        faster = fsl.FAST(in_files=tmp_infile, **{name: settings[1]})
        assert faster.cmdline == ' '.join(
            [faster.cmd, settings[0],
             "-S 1 %s" % tmp_infile])


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fast_list_outputs(setup_infile, tmpdir):
    ''' By default (no -o), FSL's fast command outputs files into the same
    directory as the input files. If the flag -o is set, it outputs files into
    the cwd '''

    def _run_and_test(opts, output_base):
        outputs = fsl.FAST(**opts)._list_outputs()
        for output in outputs.values():
            if output:
                for filename in ensure_list(output):
                    assert os.path.realpath(filename).startswith(
                        os.path.realpath(output_base))

    # set up
    tmp_infile, indir = setup_infile
    cwd = tmpdir.mkdir("new")
    cwd.chdir()
    assert indir != cwd.strpath
    out_basename = 'a_basename'

    # run and test
    opts = {'in_files': tmp_infile}
    input_path, input_filename, input_ext = split_filename(tmp_infile)
    _run_and_test(opts, os.path.join(input_path, input_filename))

    opts['out_basename'] = out_basename
    _run_and_test(opts, os.path.join(cwd.strpath, out_basename))


@pytest.fixture()
def setup_flirt(tmpdir):
    ext = Info.output_type_to_ext(Info.output_type())
    infile = tmpdir.join("infile" + ext)
    infile.open("w")
    reffile = tmpdir.join("reffile" + ext)
    reffile.open("w")
    return (tmpdir, infile.strpath, reffile.strpath)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_flirt(setup_flirt):
    # setup
    tmpdir, infile, reffile = setup_flirt

    flirter = fsl.FLIRT()
    assert flirter.cmd == 'flirt'

    flirter.inputs.bins = 256
    flirter.inputs.cost = 'mutualinfo'

    flirted = fsl.FLIRT(
        in_file=infile,
        reference=reffile,
        out_file='outfile',
        out_matrix_file='outmat.mat',
        bins=256,
        cost='mutualinfo')

    flirt_est = fsl.FLIRT(
        in_file=infile,
        reference=reffile,
        out_matrix_file='outmat.mat',
        bins=256,
        cost='mutualinfo')
    assert flirter.inputs != flirted.inputs
    assert flirted.inputs != flirt_est.inputs

    assert flirter.inputs.bins == flirted.inputs.bins
    assert flirter.inputs.cost == flirt_est.inputs.cost
    realcmd = 'flirt -in %s -ref %s -out outfile -omat outmat.mat ' \
        '-bins 256 -cost mutualinfo' % (infile, reffile)
    assert flirted.cmdline == realcmd

    flirter = fsl.FLIRT()
    # infile not specified
    with pytest.raises(ValueError):
        flirter.cmdline
    flirter.inputs.in_file = infile
    # reference not specified
    with pytest.raises(ValueError):
        flirter.cmdline
    flirter.inputs.reference = reffile

    # Generate outfile and outmatrix
    pth, fname, ext = split_filename(infile)
    outfile = fsl_name(flirter, '%s_flirt' % fname)
    outmat = '%s_flirt.mat' % fname
    realcmd = 'flirt -in %s -ref %s -out %s -omat %s' % (infile, reffile,
                                                         outfile, outmat)
    assert flirter.cmdline == realcmd

    # test apply_xfm option
    axfm = deepcopy(flirter)
    axfm.inputs.apply_xfm = True
    # in_matrix_file or uses_qform must be defined
    with pytest.raises(RuntimeError):
        axfm.cmdline
    axfm2 = deepcopy(axfm)
    # test uses_qform
    axfm.inputs.uses_qform = True
    assert axfm.cmdline == (realcmd + ' -applyxfm -usesqform')
    # test in_matrix_file
    axfm2.inputs.in_matrix_file = reffile
    assert axfm2.cmdline == (realcmd + ' -applyxfm -init %s' % reffile)

    tmpfile = tmpdir.join("file4test.nii")
    tmpfile.open("w")
    # Loop over all inputs, set a reasonable value and make sure the
    # cmdline is updated correctly.
    for key, trait_spec in sorted(fsl.FLIRT.input_spec().traits().items()):
        # Skip mandatory inputs and the trait methods
        if key in ('trait_added', 'trait_modified', 'in_file', 'reference',
                   'environ', 'output_type', 'out_file', 'out_matrix_file',
                   'in_matrix_file', 'apply_xfm',
                   'resource_monitor', 'out_log',
                   'save_log'):
            continue
        param = None
        value = None
        if key == 'args':
            param = '-v'
            value = '-v'
        elif isinstance(trait_spec.trait_type, File):
            value = tmpfile.strpath
            param = trait_spec.argstr % value
        elif trait_spec.default is False:
            param = trait_spec.argstr
            value = True
        elif key in ('searchr_x', 'searchr_y', 'searchr_z'):
            value = [-45, 45]
            param = trait_spec.argstr % ' '.join(str(elt) for elt in value)
        else:
            value = trait_spec.default
            param = trait_spec.argstr % value
        cmdline = 'flirt -in %s -ref %s' % (infile, reffile)
        # Handle autogeneration of outfile
        pth, fname, ext = split_filename(infile)
        outfile = fsl_name(fsl.FLIRT(), '%s_flirt' % fname)
        outfile = ' '.join(['-out', outfile])
        # Handle autogeneration of outmatrix
        outmatrix = '%s_flirt.mat' % fname
        outmatrix = ' '.join(['-omat', outmatrix])
        # Build command line
        cmdline = ' '.join([cmdline, outfile, outmatrix, param])
        flirter = fsl.FLIRT(in_file=infile, reference=reffile)
        setattr(flirter.inputs, key, value)
        assert flirter.cmdline == cmdline

    # Test OutputSpec
    flirter = fsl.FLIRT(in_file=infile, reference=reffile)
    pth, fname, ext = split_filename(infile)
    flirter.inputs.out_file = ''.join(['foo', ext])
    flirter.inputs.out_matrix_file = ''.join(['bar', ext])
    outs = flirter._list_outputs()
    assert outs['out_file'] == \
        os.path.join(os.getcwd(), flirter.inputs.out_file)
    assert outs['out_matrix_file'] == \
        os.path.join(os.getcwd(), flirter.inputs.out_matrix_file)
    assert not isdefined(flirter.inputs.out_log)


# Mcflirt
@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_mcflirt(setup_flirt):
    tmpdir, infile, reffile = setup_flirt

    frt = fsl.MCFLIRT()
    assert frt.cmd == 'mcflirt'
    # Test generated outfile name

    frt.inputs.in_file = infile
    _, nme = os.path.split(infile)
    outfile = os.path.join(os.getcwd(), nme)
    outfile = frt._gen_fname(outfile, suffix='_mcf')
    realcmd = 'mcflirt -in ' + infile + ' -out ' + outfile
    assert frt.cmdline == realcmd
    # Test specified outfile name
    outfile2 = '/newdata/bar.nii'
    frt.inputs.out_file = outfile2
    realcmd = 'mcflirt -in ' + infile + ' -out ' + outfile2
    assert frt.cmdline == realcmd


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_mcflirt_opt(setup_flirt):
    tmpdir, infile, reffile = setup_flirt
    _, nme = os.path.split(infile)

    opt_map = {
        'cost': ('-cost mutualinfo', 'mutualinfo'),
        'bins': ('-bins 256', 256),
        'dof': ('-dof 6', 6),
        'ref_vol': ('-refvol 2', 2),
        'scaling': ('-scaling 6.00', 6.00),
        'smooth': ('-smooth 1.00', 1.00),
        'rotation': ('-rotation 2', 2),
        'stages': ('-stages 3', 3),
        'init': ('-init %s' % (infile), infile),
        'use_gradient': ('-gdt', True),
        'use_contour': ('-edge', True),
        'mean_vol': ('-meanvol', True),
        'stats_imgs': ('-stats', True),
        'save_mats': ('-mats', True),
        'save_plots': ('-plots', True),
    }

    for name, settings in list(opt_map.items()):
        fnt = fsl.MCFLIRT(in_file=infile, **{name: settings[1]})
        outfile = os.path.join(os.getcwd(), nme)
        outfile = fnt._gen_fname(outfile, suffix='_mcf')

        instr = '-in %s' % (infile)
        outstr = '-out %s' % (outfile)
        if name in ('init', 'cost', 'dof', 'mean_vol', 'bins'):
            assert fnt.cmdline == ' '.join(
                [fnt.cmd, instr, settings[0], outstr])
        else:
            assert fnt.cmdline == ' '.join(
                [fnt.cmd, instr, outstr, settings[0]])


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_mcflirt_noinput():
    # Test error is raised when missing required args
    fnt = fsl.MCFLIRT()
    with pytest.raises(ValueError) as excinfo:
        fnt.run()
    assert str(excinfo.value).startswith(
        "MCFLIRT requires a value for input 'in_file'")


# test fnirt


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_fnirt(setup_flirt):

    tmpdir, infile, reffile = setup_flirt
    tmpdir.chdir()
    fnirt = fsl.FNIRT()
    assert fnirt.cmd == 'fnirt'

    # Test list parameters
    params = [('subsampling_scheme', '--subsamp', [4, 2, 2, 1],
               '4,2,2,1'), ('max_nonlin_iter', '--miter', [4, 4, 4, 2],
                            '4,4,4,2'), ('ref_fwhm', '--reffwhm', [4, 2, 2, 0],
                                         '4,2,2,0'), ('in_fwhm', '--infwhm',
                                                      [4, 2, 2, 0], '4,2,2,0'),
              ('apply_refmask', '--applyrefmask', [0, 0, 1, 1],
               '0,0,1,1'), ('apply_inmask', '--applyinmask', [0, 0, 0, 1],
                            '0,0,0,1'), ('regularization_lambda', '--lambda',
                                         [0.5, 0.75], '0.5,0.75'),
              ('intensity_mapping_model', '--intmod', 'global_non_linear',
               'global_non_linear')]
    for item, flag, val, strval in params:
        fnirt = fsl.FNIRT(in_file=infile, ref_file=reffile, **{item: val})
        log = fnirt._gen_fname(infile, suffix='_log.txt', change_ext=False)
        iout = fnirt._gen_fname(infile, suffix='_warped')
        if item in ('max_nonlin_iter'):
            cmd = 'fnirt --in=%s '\
                  '--logout=%s'\
                  ' %s=%s --ref=%s'\
                  ' --iout=%s' % (infile, log,
                                  flag, strval, reffile, iout)
        elif item in ('in_fwhm', 'intensity_mapping_model'):
            cmd = 'fnirt --in=%s %s=%s --logout=%s '\
                  '--ref=%s --iout=%s' % (infile, flag,
                                          strval, log, reffile, iout)
        elif item.startswith('apply'):
            cmd = 'fnirt %s=%s '\
                  '--in=%s '\
                  '--logout=%s '\
                  '--ref=%s --iout=%s' % (flag, strval,
                                          infile, log,
                                          reffile,
                                          iout)

        else:
            cmd = 'fnirt '\
                  '--in=%s --logout=%s '\
                  '--ref=%s %s=%s --iout=%s' % (infile, log,
                                                reffile,
                                                flag, strval,
                                                iout)
        assert fnirt.cmdline == cmd

    # Test ValueError is raised when missing mandatory args
    fnirt = fsl.FNIRT()
    with pytest.raises(ValueError):
        fnirt.run()
    fnirt.inputs.in_file = infile
    fnirt.inputs.ref_file = reffile
    intmap_basename = '%s_intmap' % fsl.FNIRT.intensitymap_file_basename(
        infile)
    intmap_image = fsl_name(fnirt, intmap_basename)
    intmap_txt = '%s.txt' % intmap_basename
    # doing this to create the file to pass tests for file existence
    with open(intmap_image, 'w'):
        pass
    with open(intmap_txt, 'w'):
        pass

    # test files
    opt_map = [('affine_file', '--aff=%s' % infile,
                infile), ('inwarp_file', '--inwarp=%s' % infile, infile),
               ('in_intensitymap_file', '--intin=%s' % intmap_basename,
                [intmap_image]), ('in_intensitymap_file',
                                  '--intin=%s' % intmap_basename,
                                  [intmap_image, intmap_txt]),
               ('config_file', '--config=%s' % infile,
                infile), ('refmask_file', '--refmask=%s' % infile,
                          infile), ('inmask_file', '--inmask=%s' % infile,
                                    infile), ('field_file',
                                              '--fout=%s' % infile, infile),
               ('jacobian_file', '--jout=%s' % infile,
                infile), ('modulatedref_file', '--refout=%s' % infile,
                          infile), ('out_intensitymap_file',
                                    '--intout=%s' % intmap_basename, True),
               ('out_intensitymap_file', '--intout=%s' % intmap_basename,
                intmap_image), ('fieldcoeff_file', '--cout=%s' % infile,
                                infile), ('log_file', '--logout=%s' % infile,
                                          infile)]

    for (name, settings, arg) in opt_map:
        fnirt = fsl.FNIRT(in_file=infile, ref_file=reffile, **{name: arg})

        if name in ('config_file', 'affine_file', 'field_file',
                    'fieldcoeff_file'):
            cmd = 'fnirt %s --in=%s '\
                  '--logout=%s '\
                  '--ref=%s --iout=%s' % (settings, infile, log,
                                          reffile, iout)
        elif name in ('refmask_file'):
            cmd = 'fnirt --in=%s '\
                  '--logout=%s --ref=%s '\
                  '%s '\
                  '--iout=%s' % (infile, log,
                                 reffile,
                                 settings,
                                 iout)
        elif name in ('in_intensitymap_file', 'inwarp_file', 'inmask_file',
                      'jacobian_file'):
            cmd = 'fnirt --in=%s '\
                  '%s '\
                  '--logout=%s --ref=%s '\
                  '--iout=%s' % (infile,
                                 settings,
                                 log,
                                 reffile,
                                 iout)
        elif name in ('log_file'):
            cmd = 'fnirt --in=%s '\
                  '%s --ref=%s '\
                  '--iout=%s' % (infile,
                                 settings,
                                 reffile,
                                 iout)
        else:
            cmd = 'fnirt --in=%s '\
                  '--logout=%s %s '\
                  '--ref=%s --iout=%s' % (infile, log,
                                          settings,
                                          reffile, iout)

        assert fnirt.cmdline == cmd

        if name == 'out_intensitymap_file':
            assert fnirt._list_outputs()['out_intensitymap_file'] == [
                intmap_image, intmap_txt
            ]


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_applywarp(setup_flirt):
    tmpdir, infile, reffile = setup_flirt
    opt_map = {
        'out_file': ('--out=bar.nii', 'bar.nii'),
        'premat': ('--premat=%s' % (reffile), reffile),
        'postmat': ('--postmat=%s' % (reffile), reffile),
    }

    # in_file, ref_file, field_file mandatory
    for name, settings in list(opt_map.items()):
        awarp = fsl.ApplyWarp(
            in_file=infile,
            ref_file=reffile,
            field_file=reffile,
            **{
                name: settings[1]
            })
        if name == 'out_file':
            realcmd = 'applywarp --in=%s '\
                      '--ref=%s --out=%s '\
                      '--warp=%s' % (infile, reffile,
                                     settings[1], reffile)
        else:
            outfile = awarp._gen_fname(infile, suffix='_warp')
            realcmd = 'applywarp --in=%s '\
                      '--ref=%s --out=%s '\
                      '--warp=%s %s' % (infile, reffile,
                                        outfile, reffile,
                                        settings[0])
        assert awarp.cmdline == realcmd


@pytest.fixture()
def setup_fugue(tmpdir):
    import nibabel as nb
    import numpy as np
    import os.path as op

    d = np.ones((80, 80, 80))
    infile = tmpdir.join('dumbfile.nii.gz').strpath
    nb.Nifti1Image(d, None, None).to_filename(infile)

    return (tmpdir, infile)


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
@pytest.mark.parametrize("attr, out_file", [({
    "save_unmasked_fmap": True,
    "fmap_in_file": "infile",
    "mask_file": "infile",
    "output_type": "NIFTI_GZ"
}, 'fmap_out_file'), ({
    "save_unmasked_shift": True,
    "fmap_in_file": "infile",
    "dwell_time": 1.e-3,
    "mask_file": "infile",
    "output_type": "NIFTI_GZ"
}, "shift_out_file"), ({
    "in_file": "infile",
    "mask_file": "infile",
    "shift_in_file": "infile",
    "output_type": "NIFTI_GZ"
}, 'unwarped_file')])
def test_fugue(setup_fugue, attr, out_file):
    import os.path as op
    tmpdir, infile = setup_fugue

    fugue = fsl.FUGUE()
    for key, value in attr.items():
        if value == "infile":
            setattr(fugue.inputs, key, infile)
        else:
            setattr(fugue.inputs, key, value)
    res = fugue.run()

    assert isdefined(getattr(res.outputs, out_file))
    trait_spec = fugue.inputs.trait(out_file)
    out_name = trait_spec.name_template % 'dumbfile'
    out_name += '.nii.gz'
    assert op.basename(getattr(res.outputs, out_file)) == out_name


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_first_genfname():
    first = fsl.FIRST()
    first.inputs.out_file = 'segment.nii'
    first.inputs.output_type = "NIFTI_GZ"

    value = first._gen_fname(basename='original_segmentations')
    expected_value = os.path.abspath('segment_all_fast_origsegs.nii.gz')
    assert value == expected_value
    first.inputs.method = 'none'
    value = first._gen_fname(basename='original_segmentations')
    expected_value = os.path.abspath('segment_all_none_origsegs.nii.gz')
    assert value == expected_value
    first.inputs.method = 'auto'
    first.inputs.list_of_specific_structures = ['L_Hipp', 'R_Hipp']
    value = first._gen_fname(basename='original_segmentations')
    expected_value = os.path.abspath('segment_all_none_origsegs.nii.gz')
    assert value == expected_value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import Dfs


def test_Dfs_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        curvatureWeighting=dict(
            argstr='-w %f',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputShadingVolume=dict(
            argstr='-c %s',
            extensions=None,
        ),
        inputVolumeFile=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        noNormalsFlag=dict(argstr='--nonormals', ),
        nonZeroTessellation=dict(
            argstr='-nz',
            xor=('nonZeroTessellation', 'specialTessellation'),
        ),
        outputSurfaceFile=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
        ),
        postSmoothFlag=dict(argstr='--postsmooth', ),
        scalingPercentile=dict(argstr='-f %f', ),
        smoothingConstant=dict(
            argstr='-a %f',
            usedefault=True,
        ),
        smoothingIterations=dict(
            argstr='-n %d',
            usedefault=True,
        ),
        specialTessellation=dict(
            argstr='%s',
            position=-1,
            requires=['tessellationThreshold'],
            xor=('nonZeroTessellation', 'specialTessellation'),
        ),
        tessellationThreshold=dict(argstr='%f', ),
        timer=dict(argstr='--timer', ),
        verbosity=dict(argstr='-v %d', ),
        zeroPadFlag=dict(argstr='-z', ),
    )
    inputs = Dfs.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Dfs_outputs():
    output_map = dict(outputSurfaceFile=dict(extensions=None, ), )
    outputs = Dfs.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import OneSampleTTest


def test_OneSampleTTest_inputs():
    input_map = dict(
        allow_ill_cond=dict(argstr='--illcond', ),
        allow_repeated_subjects=dict(argstr='--allowsubjrep', ),
        args=dict(argstr='%s', ),
        calc_AR1=dict(argstr='--tar1', ),
        check_opts=dict(argstr='--checkopts', ),
        compute_log_y=dict(argstr='--logy', ),
        contrast=dict(argstr='--C %s...', ),
        cortex=dict(
            argstr='--cortex',
            xor=['label_file'],
        ),
        debug=dict(argstr='--debug', ),
        design=dict(
            argstr='--X %s',
            extensions=None,
            xor=('fsgd', 'design', 'one_sample'),
        ),
        diag=dict(argstr='--diag %d', ),
        diag_cluster=dict(argstr='--diag-cluster', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_fx_dof=dict(
            argstr='--ffxdof %d',
            xor=['fixed_fx_dof_file'],
        ),
        fixed_fx_dof_file=dict(
            argstr='--ffxdofdat %d',
            extensions=None,
            xor=['fixed_fx_dof'],
        ),
        fixed_fx_var=dict(
            argstr='--yffxvar %s',
            extensions=None,
        ),
        force_perm=dict(argstr='--perm-force', ),
        fsgd=dict(
            argstr='--fsgd %s %s',
            xor=('fsgd', 'design', 'one_sample'),
        ),
        fwhm=dict(argstr='--fwhm %f', ),
        glm_dir=dict(
            argstr='--glmdir %s',
            genfile=True,
        ),
        hemi=dict(),
        in_file=dict(
            argstr='--y %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        invert_mask=dict(argstr='--mask-inv', ),
        label_file=dict(
            argstr='--label %s',
            extensions=None,
            xor=['cortex'],
        ),
        mask_file=dict(
            argstr='--mask %s',
            extensions=None,
        ),
        no_contrast_ok=dict(argstr='--no-contrasts-ok', ),
        no_est_fwhm=dict(argstr='--no-est-fwhm', ),
        no_mask_smooth=dict(argstr='--no-mask-smooth', ),
        no_prune=dict(
            argstr='--no-prune',
            xor=['prunethresh'],
        ),
        one_sample=dict(
            argstr='--osgm',
            xor=('one_sample', 'fsgd', 'design', 'contrast'),
        ),
        pca=dict(argstr='--pca', ),
        per_voxel_reg=dict(argstr='--pvr %s...', ),
        profile=dict(argstr='--profile %d', ),
        prune=dict(argstr='--prune', ),
        prune_thresh=dict(
            argstr='--prune_thr %f',
            xor=['noprune'],
        ),
        resynth_test=dict(argstr='--resynthtest %d', ),
        save_cond=dict(argstr='--save-cond', ),
        save_estimate=dict(argstr='--yhat-save', ),
        save_res_corr_mtx=dict(argstr='--eres-scm', ),
        save_residual=dict(argstr='--eres-save', ),
        seed=dict(argstr='--seed %d', ),
        self_reg=dict(argstr='--selfreg %d %d %d', ),
        sim_done_file=dict(
            argstr='--sim-done %s',
            extensions=None,
        ),
        sim_sign=dict(argstr='--sim-sign %s', ),
        simulation=dict(argstr='--sim %s %d %f %s', ),
        subject_id=dict(),
        subjects_dir=dict(),
        surf=dict(
            argstr='--surf %s %s %s',
            requires=['subject_id', 'hemi'],
        ),
        surf_geo=dict(usedefault=True, ),
        synth=dict(argstr='--synth', ),
        uniform=dict(argstr='--uniform %f %f', ),
        var_fwhm=dict(argstr='--var-fwhm %f', ),
        vox_dump=dict(argstr='--voxdump %d %d %d', ),
        weight_file=dict(
            extensions=None,
            xor=['weighted_ls'],
        ),
        weight_inv=dict(
            argstr='--w-inv',
            xor=['weighted_ls'],
        ),
        weight_sqrt=dict(
            argstr='--w-sqrt',
            xor=['weighted_ls'],
        ),
        weighted_ls=dict(
            argstr='--wls %s',
            extensions=None,
            xor=('weight_file', 'weight_inv', 'weight_sqrt'),
        ),
    )
    inputs = OneSampleTTest.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_OneSampleTTest_outputs():
    output_map = dict(
        beta_file=dict(extensions=None, ),
        dof_file=dict(extensions=None, ),
        error_file=dict(extensions=None, ),
        error_stddev_file=dict(extensions=None, ),
        error_var_file=dict(extensions=None, ),
        estimate_file=dict(extensions=None, ),
        frame_eigenvectors=dict(extensions=None, ),
        ftest_file=dict(),
        fwhm_file=dict(extensions=None, ),
        gamma_file=dict(),
        gamma_var_file=dict(),
        glm_dir=dict(),
        mask_file=dict(extensions=None, ),
        sig_file=dict(),
        singular_values=dict(extensions=None, ),
        spatial_eigenvectors=dict(extensions=None, ),
        svd_stats_file=dict(extensions=None, ),
    )
    outputs = OneSampleTTest.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import FIRST


def test_FIRST_inputs():
    input_map = dict(
        affine_file=dict(
            argstr='-a %s',
            extensions=None,
            position=6,
        ),
        args=dict(argstr='%s', ),
        brain_extracted=dict(
            argstr='-b',
            position=2,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        list_of_specific_structures=dict(
            argstr='-s %s',
            position=5,
            sep=',',
        ),
        method=dict(
            argstr='-m %s',
            position=4,
            usedefault=True,
            xor=['method_as_numerical_threshold'],
        ),
        method_as_numerical_threshold=dict(
            argstr='-m %.4f',
            position=4,
        ),
        no_cleanup=dict(
            argstr='-d',
            position=3,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            hash_files=False,
            mandatory=True,
            position=-1,
            usedefault=True,
        ),
        output_type=dict(),
        verbose=dict(
            argstr='-v',
            position=1,
        ),
    )
    inputs = FIRST.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FIRST_outputs():
    output_map = dict(
        bvars=dict(),
        original_segmentations=dict(extensions=None, ),
        segmentation_file=dict(extensions=None, ),
        vtk_surfaces=dict(),
    )
    outputs = FIRST.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import MCFLIRT


def test_MCFLIRT_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bins=dict(argstr='-bins %d', ),
        cost=dict(argstr='-cost %s', ),
        dof=dict(argstr='-dof %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-in %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        init=dict(
            argstr='-init %s',
            extensions=None,
        ),
        interpolation=dict(argstr='-%s_final', ),
        mean_vol=dict(argstr='-meanvol', ),
        out_file=dict(
            argstr='-out %s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_type=dict(),
        ref_file=dict(
            argstr='-reffile %s',
            extensions=None,
        ),
        ref_vol=dict(argstr='-refvol %d', ),
        rotation=dict(argstr='-rotation %d', ),
        save_mats=dict(argstr='-mats', ),
        save_plots=dict(argstr='-plots', ),
        save_rms=dict(argstr='-rmsabs -rmsrel', ),
        scaling=dict(argstr='-scaling %.2f', ),
        smooth=dict(argstr='-smooth %.2f', ),
        stages=dict(argstr='-stages %d', ),
        stats_imgs=dict(argstr='-stats', ),
        use_contour=dict(argstr='-edge', ),
        use_gradient=dict(argstr='-gdt', ),
    )
    inputs = MCFLIRT.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MCFLIRT_outputs():
    output_map = dict(
        mat_file=dict(),
        mean_img=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        par_file=dict(extensions=None, ),
        rms_files=dict(),
        std_img=dict(extensions=None, ),
        variance_img=dict(extensions=None, ),
    )
    outputs = MCFLIRT.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Unifize


def test_Unifize_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        cl_frac=dict(argstr='-clfrac %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi=dict(
            argstr='-EPI',
            requires=['no_duplo', 't2'],
            xor=['gm'],
        ),
        gm=dict(argstr='-GM', ),
        in_file=dict(
            argstr='-input %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        no_duplo=dict(argstr='-noduplo', ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source='in_file',
            name_template='%s_unifized',
        ),
        outputtype=dict(),
        quiet=dict(argstr='-quiet', ),
        rbt=dict(argstr='-rbt %f %f %f', ),
        scale_file=dict(
            argstr='-ssave %s',
            extensions=None,
        ),
        t2=dict(argstr='-T2', ),
        t2_up=dict(argstr='-T2up %f', ),
        urad=dict(argstr='-Urad %s', ),
    )
    inputs = Unifize.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Unifize_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        scale_file=dict(extensions=None, ),
    )
    outputs = Unifize.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..epi import EddyQuad


def test_EddyQuad_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        base_name=dict(
            argstr='%s',
            position=0,
            usedefault=True,
        ),
        bval_file=dict(
            argstr='--bvals %s',
            extensions=None,
            mandatory=True,
        ),
        bvec_file=dict(
            argstr='--bvecs %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        field=dict(
            argstr='--field %s',
            extensions=None,
        ),
        idx_file=dict(
            argstr='--eddyIdx %s',
            extensions=None,
            mandatory=True,
        ),
        mask_file=dict(
            argstr='--mask %s',
            extensions=None,
            mandatory=True,
        ),
        output_dir=dict(
            argstr='--output-dir %s',
            name_source=['base_name'],
            name_template='%s.qc',
        ),
        output_type=dict(),
        param_file=dict(
            argstr='--eddyParams %s',
            extensions=None,
            mandatory=True,
        ),
        slice_spec=dict(
            argstr='--slspec %s',
            extensions=None,
        ),
        verbose=dict(argstr='--verbose', ),
    )
    inputs = EddyQuad.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_EddyQuad_outputs():
    output_map = dict(
        avg_b0_pe_png=dict(),
        avg_b_png=dict(),
        clean_volumes=dict(extensions=None, ),
        cnr_png=dict(),
        qc_json=dict(extensions=None, ),
        qc_pdf=dict(extensions=None, ),
        residuals=dict(extensions=None, ),
        vdm_png=dict(extensions=None, ),
    )
    outputs = EddyQuad.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import division

import numpy as np

import numpy.testing as npt
from .. import rapidart as ra
from ...interfaces.base import Bunch


def test_ad_init():
    ad = ra.ArtifactDetect(use_differences=[True, False])
    assert ad.inputs.use_differences[0]
    assert not ad.inputs.use_differences[1]


def test_ad_output_filenames():
    ad = ra.ArtifactDetect()
    outputdir = '/tmp'
    f = 'motion.nii'
    (outlierfile, intensityfile, statsfile, normfile, plotfile,
     displacementfile, maskfile) = ad._get_output_filenames(f, outputdir)
    assert outlierfile == '/tmp/art.motion_outliers.txt'
    assert intensityfile == '/tmp/global_intensity.motion.txt'
    assert statsfile == '/tmp/stats.motion.txt'
    assert normfile == '/tmp/norm.motion.txt'
    assert plotfile == '/tmp/plot.motion.png'
    assert displacementfile == '/tmp/disp.motion.nii'
    assert maskfile == '/tmp/mask.motion.nii'


def test_ad_get_affine_matrix():
    matrix = ra._get_affine_matrix(np.array([0]), 'SPM')
    npt.assert_equal(matrix, np.eye(4))
    # test translation
    params = [1, 2, 3]
    matrix = ra._get_affine_matrix(params, 'SPM')
    out = np.eye(4)
    out[0:3, 3] = params
    npt.assert_equal(matrix, out)
    # test rotation
    params = np.array([0, 0, 0, np.pi / 2, np.pi / 2, np.pi / 2])
    matrix = ra._get_affine_matrix(params, 'SPM')
    out = np.array([0, 0, 1, 0, 0, -1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1]).reshape(
        (4, 4))
    npt.assert_almost_equal(matrix, out)
    # test scaling
    params = np.array([0, 0, 0, 0, 0, 0, 1, 2, 3])
    matrix = ra._get_affine_matrix(params, 'SPM')
    out = np.array([1, 0, 0, 0, 0, 2, 0, 0, 0, 0, 3, 0, 0, 0, 0, 1]).reshape(
        (4, 4))
    npt.assert_equal(matrix, out)
    # test shear
    params = np.array([0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 3])
    matrix = ra._get_affine_matrix(params, 'SPM')
    out = np.array([1, 1, 2, 0, 0, 1, 3, 0, 0, 0, 1, 0, 0, 0, 0, 1]).reshape(
        (4, 4))
    npt.assert_equal(matrix, out)


def test_ad_get_norm():
    params = np.array([
        0, 0, 0, 0, 0, 0, 0, 0, 0, np.pi / 4, np.pi / 4, np.pi / 4, 0, 0, 0,
        -np.pi / 4, -np.pi / 4, -np.pi / 4
    ]).reshape((3, 6))
    norm, _ = ra._calc_norm(params, False, 'SPM')
    npt.assert_almost_equal(norm,
                            np.array([18.86436316, 37.74610158, 31.29780829]))
    norm, _ = ra._calc_norm(params, True, 'SPM')
    npt.assert_almost_equal(norm, np.array([0., 143.72192614, 173.92527131]))


def test_sc_init():
    sc = ra.StimulusCorrelation(concatenated_design=True)
    assert sc.inputs.concatenated_design


def test_sc_populate_inputs():
    sc = ra.StimulusCorrelation()
    inputs = Bunch(
        realignment_parameters=None,
        intensity_values=None,
        spm_mat_file=None,
        concatenated_design=None)
    assert set(sc.inputs.__dict__.keys()) == set(inputs.__dict__.keys())


def test_sc_output_filenames():
    sc = ra.StimulusCorrelation()
    outputdir = '/tmp'
    f = 'motion.nii'
    corrfile = sc._get_output_filenames(f, outputdir)
    assert corrfile == '/tmp/qa.motion_stimcorr.txt'



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import SliceTimer


def test_SliceTimer_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        custom_order=dict(
            argstr='--ocustom=%s',
            extensions=None,
        ),
        custom_timings=dict(
            argstr='--tcustom=%s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        global_shift=dict(argstr='--tglobal', ),
        in_file=dict(
            argstr='--in=%s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        index_dir=dict(argstr='--down', ),
        interleaved=dict(argstr='--odd', ),
        out_file=dict(
            argstr='--out=%s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_type=dict(),
        slice_direction=dict(argstr='--direction=%d', ),
        time_repetition=dict(argstr='--repeat=%f', ),
    )
    inputs = SliceTimer.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SliceTimer_outputs():
    output_map = dict(slice_time_corrected_file=dict(extensions=None, ), )
    outputs = SliceTimer.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..aroma import ICA_AROMA


def test_ICA_AROMA_inputs():
    input_map = dict(
        TR=dict(argstr='-tr %.3f', ),
        args=dict(argstr='%s', ),
        denoise_type=dict(
            argstr='-den %s',
            mandatory=True,
            usedefault=True,
        ),
        dim=dict(argstr='-dim %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        feat_dir=dict(
            argstr='-feat %s',
            mandatory=True,
            xor=[
                'in_file', 'mat_file', 'fnirt_warp_file', 'motion_parameters'
            ],
        ),
        fnirt_warp_file=dict(
            argstr='-warp %s',
            extensions=None,
            xor=['feat_dir'],
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            xor=['feat_dir'],
        ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
            xor=['feat_dir'],
        ),
        mat_file=dict(
            argstr='-affmat %s',
            extensions=None,
            xor=['feat_dir'],
        ),
        melodic_dir=dict(argstr='-meldir %s', ),
        motion_parameters=dict(
            argstr='-mc %s',
            extensions=None,
            mandatory=True,
            xor=['feat_dir'],
        ),
        out_dir=dict(
            argstr='-o %s',
            mandatory=True,
            usedefault=True,
        ),
    )
    inputs = ICA_AROMA.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ICA_AROMA_outputs():
    output_map = dict(
        aggr_denoised_file=dict(extensions=None, ),
        nonaggr_denoised_file=dict(extensions=None, ),
        out_dir=dict(),
    )
    outputs = ICA_AROMA.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..io import MySQLSink


def test_MySQLSink_inputs():
    input_map = dict(
        config=dict(
            extensions=None,
            mandatory=True,
            xor=['host'],
        ),
        database_name=dict(mandatory=True, ),
        host=dict(
            mandatory=True,
            requires=['username', 'password'],
            usedefault=True,
            xor=['config'],
        ),
        password=dict(),
        table_name=dict(mandatory=True, ),
        username=dict(),
    )
    inputs = MySQLSink.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..cifti import CiftiSmooth


def test_CiftiSmooth_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        cerebellum_corrected_areas=dict(
            argstr='cerebellum-corrected-areas %s',
            extensions=None,
            position=10,
            requires=['cerebellum_surf'],
        ),
        cerebellum_surf=dict(
            argstr='-cerebellum-surface %s',
            extensions=None,
            position=9,
        ),
        cifti_roi=dict(
            argstr='-cifti-roi %s',
            extensions=None,
            position=11,
        ),
        direction=dict(
            argstr='%s',
            mandatory=True,
            position=3,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fix_zeros_surf=dict(
            argstr='-fix-zeros-surface',
            position=13,
        ),
        fix_zeros_vol=dict(
            argstr='-fix-zeros-volume',
            position=12,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        left_corrected_areas=dict(
            argstr='-left-corrected-areas %s',
            extensions=None,
            position=6,
        ),
        left_surf=dict(
            argstr='-left-surface %s',
            extensions=None,
            mandatory=True,
            position=5,
        ),
        merged_volume=dict(
            argstr='-merged-volume',
            position=14,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            keep_extension=True,
            name_source=['in_file'],
            name_template='smoothed_%s.nii',
            position=4,
        ),
        right_corrected_areas=dict(
            argstr='-right-corrected-areas %s',
            extensions=None,
            position=8,
        ),
        right_surf=dict(
            argstr='-right-surface %s',
            extensions=None,
            mandatory=True,
            position=7,
        ),
        sigma_surf=dict(
            argstr='%s',
            mandatory=True,
            position=1,
        ),
        sigma_vol=dict(
            argstr='%s',
            mandatory=True,
            position=2,
        ),
    )
    inputs = CiftiSmooth.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CiftiSmooth_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = CiftiSmooth.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..odf import ODFRecon


def test_ODFRecon_inputs():
    input_map = dict(
        DWI=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        args=dict(argstr='%s', ),
        dsi=dict(argstr='-dsi', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filter=dict(argstr='-f', ),
        image_orientation_vectors=dict(argstr='-iop %f', ),
        matrix=dict(
            argstr='-mat %s',
            extensions=None,
            mandatory=True,
        ),
        n_b0=dict(
            argstr='-b0 %s',
            mandatory=True,
        ),
        n_directions=dict(
            argstr='%s',
            mandatory=True,
            position=2,
        ),
        n_output_directions=dict(
            argstr='%s',
            mandatory=True,
            position=3,
        ),
        oblique_correction=dict(argstr='-oc', ),
        out_prefix=dict(
            argstr='%s',
            position=4,
            usedefault=True,
        ),
        output_entropy=dict(argstr='-oe', ),
        output_type=dict(
            argstr='-ot %s',
            usedefault=True,
        ),
        sharpness=dict(argstr='-s %f', ),
        subtract_background=dict(argstr='-bg', ),
    )
    inputs = ODFRecon.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ODFRecon_outputs():
    output_map = dict(
        B0=dict(extensions=None, ),
        DWI=dict(extensions=None, ),
        ODF=dict(extensions=None, ),
        entropy=dict(extensions=None, ),
        max=dict(extensions=None, ),
    )
    outputs = ODFRecon.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..segmentation import DenoiseImage


def test_DenoiseImage_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        dimension=dict(argstr='-d %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_image=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        noise_image=dict(
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=['input_image'],
            name_template='%s_noise',
        ),
        noise_model=dict(
            argstr='-n %s',
            usedefault=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        output_image=dict(
            argstr='-o %s',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source=['input_image'],
            name_template='%s_noise_corrected',
        ),
        save_noise=dict(
            mandatory=True,
            usedefault=True,
            xor=['noise_image'],
        ),
        shrink_factor=dict(
            argstr='-s %s',
            usedefault=True,
        ),
        verbose=dict(argstr='-v', ),
    )
    inputs = DenoiseImage.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DenoiseImage_outputs():
    output_map = dict(
        noise_image=dict(extensions=None, ),
        output_image=dict(extensions=None, ),
    )
    outputs = DenoiseImage.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..regutils import RegTransform


def test_RegTransform_inputs():
    input_map = dict(
        aff_2_rig_input=dict(
            argstr='-aff2rig %s',
            extensions=None,
            position=-2,
            xor=[
                'def_input', 'disp_input', 'flow_input', 'comp_input',
                'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input',
                'half_input', 'make_aff_input', 'flirt_2_nr_input'
            ],
        ),
        args=dict(argstr='%s', ),
        comp_input=dict(
            argstr='-comp %s',
            extensions=None,
            position=-3,
            requires=['comp_input2'],
            xor=[
                'def_input', 'disp_input', 'flow_input', 'upd_s_form_input',
                'inv_aff_input', 'inv_nrr_input', 'half_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        comp_input2=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        def_input=dict(
            argstr='-def %s',
            extensions=None,
            position=-2,
            xor=[
                'disp_input', 'flow_input', 'comp_input', 'upd_s_form_input',
                'inv_aff_input', 'inv_nrr_input', 'half_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        disp_input=dict(
            argstr='-disp %s',
            extensions=None,
            position=-2,
            xor=[
                'def_input', 'flow_input', 'comp_input', 'upd_s_form_input',
                'inv_aff_input', 'inv_nrr_input', 'half_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        flirt_2_nr_input=dict(
            argstr='-flirtAff2NR %s %s %s',
            position=-2,
            xor=[
                'def_input', 'disp_input', 'flow_input', 'comp_input',
                'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input',
                'half_input', 'make_aff_input', 'aff_2_rig_input'
            ],
        ),
        flow_input=dict(
            argstr='-flow %s',
            extensions=None,
            position=-2,
            xor=[
                'def_input', 'disp_input', 'comp_input', 'upd_s_form_input',
                'inv_aff_input', 'inv_nrr_input', 'half_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        half_input=dict(
            argstr='-half %s',
            extensions=None,
            position=-2,
            xor=[
                'def_input', 'disp_input', 'flow_input', 'comp_input',
                'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        inv_aff_input=dict(
            argstr='-invAff %s',
            extensions=None,
            position=-2,
            xor=[
                'def_input', 'disp_input', 'flow_input', 'comp_input',
                'upd_s_form_input', 'inv_nrr_input', 'half_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        inv_nrr_input=dict(
            argstr='-invNrr %s %s',
            position=-2,
            xor=[
                'def_input', 'disp_input', 'flow_input', 'comp_input',
                'upd_s_form_input', 'inv_aff_input', 'half_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        make_aff_input=dict(
            argstr='-makeAff %f %f %f %f %f %f %f %f %f %f %f %f',
            position=-2,
            xor=[
                'def_input', 'disp_input', 'flow_input', 'comp_input',
                'upd_s_form_input', 'inv_aff_input', 'inv_nrr_input',
                'half_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        omp_core_val=dict(
            argstr='-omp %i',
            usedefault=True,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        ref1_file=dict(
            argstr='-ref %s',
            extensions=None,
            position=0,
        ),
        ref2_file=dict(
            argstr='-ref2 %s',
            extensions=None,
            position=1,
            requires=['ref1_file'],
        ),
        upd_s_form_input=dict(
            argstr='-updSform %s',
            extensions=None,
            position=-3,
            requires=['upd_s_form_input2'],
            xor=[
                'def_input', 'disp_input', 'flow_input', 'comp_input',
                'inv_aff_input', 'inv_nrr_input', 'half_input',
                'make_aff_input', 'aff_2_rig_input', 'flirt_2_nr_input'
            ],
        ),
        upd_s_form_input2=dict(
            argstr='%s',
            extensions=None,
            position=-2,
            requires=['upd_s_form_input'],
        ),
    )
    inputs = RegTransform.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RegTransform_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = RegTransform.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..developer import MedicAlgorithmMipavReorient


def test_MedicAlgorithmMipavReorient_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inInterpolation=dict(argstr='--inInterpolation %s', ),
        inNew=dict(argstr='--inNew %s', ),
        inResolution=dict(argstr='--inResolution %s', ),
        inSource=dict(
            argstr='--inSource %s',
            sep=';',
        ),
        inTemplate=dict(
            argstr='--inTemplate %s',
            extensions=None,
        ),
        inUser=dict(argstr='--inUser %s', ),
        inUser2=dict(argstr='--inUser2 %s', ),
        inUser3=dict(argstr='--inUser3 %s', ),
        inUser4=dict(argstr='--inUser4 %s', ),
        null=dict(argstr='--null %s', ),
        outReoriented=dict(
            argstr='--outReoriented %s',
            sep=';',
        ),
        xDefaultMem=dict(argstr='-xDefaultMem %d', ),
        xMaxProcess=dict(
            argstr='-xMaxProcess %d',
            usedefault=True,
        ),
        xPrefExt=dict(argstr='--xPrefExt %s', ),
    )
    inputs = MedicAlgorithmMipavReorient.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MedicAlgorithmMipavReorient_outputs():
    output_map = dict()
    outputs = MedicAlgorithmMipavReorient.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..diffusion import DiffusionWeightedVolumeMasking


def test_DiffusionWeightedVolumeMasking_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputVolume=dict(
            argstr='%s',
            extensions=None,
            position=-4,
        ),
        otsuomegathreshold=dict(argstr='--otsuomegathreshold %f', ),
        outputBaseline=dict(
            argstr='%s',
            hash_files=False,
            position=-2,
        ),
        removeislands=dict(argstr='--removeislands ', ),
        thresholdMask=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
    )
    inputs = DiffusionWeightedVolumeMasking.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DiffusionWeightedVolumeMasking_outputs():
    output_map = dict(
        outputBaseline=dict(
            extensions=None,
            position=-2,
        ),
        thresholdMask=dict(
            extensions=None,
            position=-1,
        ),
    )
    outputs = DiffusionWeightedVolumeMasking.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
